package au.com.fourtel.info.clock;

import au.com.fourtel.info.support.exceptions.ClockException;
import java.net.URL;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.ResourceBundle;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import javafx.animation.Timeline;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Button;
import javafx.scene.control.Label;

public class FXMLController ////  implements Initializable 
{

		//<editor-fold defaultstate="collapsed" desc="private members ...">

	private static final ClockEnvironment	CLOCK_ENVIRONMENT = new ClockEnvironment();

    private static final Integer STARTTIME = 15;
    private Timeline timeline;
    private Integer  timeSeconds = STARTTIME;

		//</editor-fold>

        //<editor-fold defaultstate="collapsed" desc="FXML injection controls ...">

    @FXML
    private ResourceBundle resources;

    @FXML
    private URL location;

    @FXML
    private Label digitalClock;

    @FXML
    private Button button;

    @FXML
    private Label label;

    //</editor-fold>

    
    @FXML
    private void handleButtonAction(ActionEvent event) {
        System.out.println("You clicked me!");
        label.setText("Hello World!");
        digitalClock.setText( fmtTime() );

		startClock();
    }

	    //<editor-fold defaultstate="collapsed" desc="constructors and initialisation ... ">

	public FXMLController()
	{

	}



    @FXML
    void initialize() 
    {
        assert location     != null : "@FXML \"location\" URL was not injected: check MainApp.java.";
        assert resources    != null : "fx:id=\"resources\" ResourceBundle was not injected: check MainApp.java.";
        
        assert digitalClock != null : "fx:id=\"clock\" was not injected: check your FXML file 'scene.fxml'.";
        assert button       != null : "fx:id=\"button\" was not injected: check your FXML file 'scene.fxml'.";
        assert label        != null : "fx:id=\"label\" was not injected: check your FXML file 'scene.fxml'.";

    }

//    @Override
//    public void initialize(URL url, ResourceBundle rb) 
//    {
//        initialize();
//        // TODO
//    }

			//</editor-fold>

        /**
         * Formate time:
         * @return HH:mm:ss
         */
    public String fmtTime()
    {
        String rslt;
        Calendar calendar = GregorianCalendar.getInstance();
        int hour = calendar.get(Calendar.HOUR);
        int min  = calendar.get(Calendar.MINUTE);
        int sec  = calendar.get(Calendar.SECOND);

        rslt = String.format(  "%02d:%02d:%02d", hour, min, sec );
        return rslt;
    }


    ///////////////////////////////
    //  -- http://docs.oracle.com/javafx/2/events/KeyboardExample.java.htm
    //  -- http://docs.oracle.com/javafx/2/api/javafx/event/EventHandler.html
    //  -- http://docs.oracle.com/javase/7/docs/api/java/beans/EventHandler.html
    //
    //  -- http://stackoverflow.com/questions/16764549/timers-and-javafx
    //  -- http://stackoverflow.com/questions/20268593/timertask-cant-update-the-javafx-label-text
    //  -- http://www.asgteach.com/blog/?p=334#Listing2
    //  -- http://stackoverflow.com/questions/16128423/how-to-update-the-label-box-every-2-seconds-in-java-fx
    ///////////////////////////////


		/**
		 *  Starts up clock update process
		 *
		 *    1. Kick-off clock
		 *    2. Hosts "clock state" object: ClockEnvironment (class global).
		 *    3. A clock-ticking class to run every N second(s).
		 *    4.   Schedule (very short) GUI clock-update task.
		 *    5.   Schedule (very short) clock-"off" message task.
		 *    6. Provide clock shut-down method to stop clock
		 *    7. Provide clock restart method to start clock (after stop)
		 *    8. Keep the number of tasks MINIMAL and SHORT.
		 *
		 *  Everything should be run using the JavaFX platform scheduling
		 *  That way we can avoid critical sections because only one-thing
		 *  is run at a time.  Implications for tasks:
		 *
		 *   * Keep it simple.
		 *   * NO long-running tasks, use a parallel scheduling mechanism
		 *   * Keep it short rely on JavaFX to do keep the repeat pulse beat.
		 *
		 */

		 ////////////
		 //
		 //  Start Clock
		 //
		 //  @see
		 //		-- http://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ScheduledThreadPoolExecutor.html#scheduleAtFixedRate%28java.lang.Runnable,%20long,%20long,%20java.util.concurrent.TimeUnit%29
		 //
    public void startClock()
    {
		final int INITIAL_DELAY =  1;	//-- let it synchronise with system clock (TODO: tune this)
		final int REPEAT_PERIOD =  1;

		final ScheduledExecutorService	scheduler  = Executors.newScheduledThreadPool( 1 );
		final TickingClock				clockTicks = new TickingClock();

		scheduler.scheduleAtFixedRate(
			clockTicks,
			INITIAL_DELAY,
			REPEAT_PERIOD, TimeUnit.SECONDS);
    }

		/**
		 *  Information block about clock and anything
		 *  to be shared with clock.
		 */
	private static  class ClockEnvironment
	{
		/**
		 *  @see
		 *		-- http://people.clarkson.edu/~jsearlem/cs242/fa13/lectures/08.javaFX.pdf (PDF)
		 *		-- http://stackoverflow.com/questions/16719503/what-to-do-about-uncaught-run-time-exceptions-in-java-fx2
		 *		-- http://docs.oracle.com/javafx/2/threads/jfxpub-threads.htm
		 */

		private boolean clock_running = true;

		public ClockEnvironment()
		{
		}

			//-- Avoid critical section by ensuring
			//   these are called ONLY within the
			//   JavaFX platform
			//
		public void restartClock()
		{
			if( Platform.isFxApplicationThread() ){
				clock_running = true;
			}
			else
			{
				throw new ClockException( "Ilegal call to ClockEnvironment::restartClock() " );
			}
		}

		public void stopClock()
		{
			if( Platform.isFxApplicationThread() ){
				clock_running = false;
			}
			else
			{
				throw new ClockException();// "Ilegal call to ClockEnvironment::stopClock() ");
			}
		}

	}//clockEnvironment class



	////////////////////
	//  Clock process:
	//
    ////  -- http://stackoverflow.com/questions/16764549/timers-and-javafx
    //    -- http://stackoverflow.com/questions/16128423/how-to-update-the-label-box-every-2-seconds-in-java-fx
    //    -- http://docs.oracle.com/javafx/2/events/handlers.htm
    //    -- http://docs.oracle.com/javafx/2/events/KeyboardExample.java.htm
    //    -- http://docs.oracle.com/javase/7/docs/api/java/beans/EventHandler.html
    //    --

	private class TickingClock extends Thread
	{
		public TickingClock()
		{
		}

			/**
			 *  Clock pulse logic here.
			 *
			 *  UI updates are fired separately on the JavaFX
			 *  event queue to keep things in-order.
			 *
			 */
		@Override
		public void run()
		{
			ClockRefresh refreshClock;
			refreshClock = new ClockRefresh();
			Platform.runLater( refreshClock );
		}

	}//TickingClock

	private class ClockRefresh extends Thread
	{
		public ClockRefresh()
		{
		}

			/**
			 *  Clock update logic here.
			 *
			 *  Just update the display
			 *
			 */
		@Override
		public void run()
		{
			digitalClock.setText( fmtTime() );
		}

	}//ClockRefresh class


}//FXMLController class
